# Utilities for working with macros

# Jon K. Peck

# history
# 10-sep-2019 original version
# 23-sep-2019 adjust for KO macros subdirectory
# 14-jan-2020 add insertMacroFiles function
# 04-dec-2022 hide Notes table

import spss, spssaux, os.path, random
    
defaultdir = "KO_macros"    # assumed directory or subdirectory.  No drive letter
handlename = "KOmacros"     # handle that will be defined to point to macro location

def checkMacroDefinitions(macroname):
    """return True if macroname is defined
    
    macroname is the case insensitive name of the macro to check for
    
    If the macro is already defined, no action is taken to define the file handle"""
    
    try:
        rn = str(random.uniform(.05, 1))
        spss.Submit("""OMS SELECT ALL /DESTINATION VIEWER=NO /TAG="%s".""" % rn)
        tag = spssaux.CreateXMLOutput("display macros", omsid='File Information', subtype='Macro List')[0]
        macros = spssaux.getValuesFromXmlWorkspace(tag, "Macro List", xpathExpr=False)
        spss.DeleteXPathHandle(tag)
        macros = [item.strip().lower() for item in macros]
    finally:
        spss.Submit("""OMSEND TAG="%s".""" % rn)
    return macroname.lower() in macros

def locateMacroFile(macrofile):
    """find macro file if any looking in KO macros or SHOW EXT locations and KO macros subdirectory
    
    macrofile is the filename to find (including extension)"""
    
    tryname = "C:" + os.path.sep + defaultdir + os.path.sep + macrofile
    if os.path.exists(tryname) and os.path.isfile(tryname):
        dohandle("C:" + os.path.sep + defaultdir)
        return tryname
    
    # search SHOW EXT locations but screening out dialog and R parts
    locations = getlocations()

    for loc in locations:
        tryname = loc + os.path.sep + macrofile
        if os.path.exists(tryname) and os.path.isfile(tryname):
            dohandle(loc)
            return tryname
        else:
            # check a subdirectory
            tryname = loc + os.path.sep + defaultdir + os.path.sep + macrofile
            if os.path.exists(tryname) and os.path.isfile(tryname):
                dohandle(loc + os.path.sep + defaultdir)
                return tryname
    raise ValueError("""Macro file: %s not found""" % macrofile)

def insertMacroFile(macroname, macrofile, reload=False):
    """Locate macrofile and execute an INSERT if found if not already defined
    
    macroname is the macro name to be defined
    macrofile is the file to insert, including extension to define it
    reload specifies to reload even if already defined"""
    
    if checkMacroDefinitions(macroname) and not reload:
        return
    f = locateMacroFile(macrofile)    
    spss.Submit('''INSERT FILE="%s".''' % f)
    
# usage:
#   insertMacrofiles([(macro1, macrofile), (macro2, macrofile), ...], reload)
def insertMacroFiles(maclist, reload=False):
    """Locate macrofiles and execute an INSERT if found if not already defined for each
    
    maclist is a sequence of duples where each duple consists of a macro name and macrofile
    reload specifies to reload even if already defined
    
    If a specified macro file is not found, an exception is raised, and processing stops"""
    
    for macroname, macrofile in maclist:
        insertMacroFile(macroname, macrofile, reload)

    
        
        
def dohandle(location):
    """define file handle KOmacros pointing to location
    
    location is the path without trailing separator for the handle"""
    
    fh = spssaux.FileHandles()
    
    # handle is always redefined, because definition might have changed
    # since a previous definition
    fh.createHandle(handlename, location)
    
def getlocations():
    """return list of valid locations for extension commands"""

    xpath = "//group[2]/category//cell/@text"
    
    tag, ignore = spssaux.createXmlOutput("SHOW EXT.", "SHOW")
    # need to screen out blank or irrelevant entries
    locs = [item for item in spss.EvaluateXPath(tag, '/', xpath) if item != " " and (item.__contains__("/") or item.__contains__("\\"))]
    spss.DeleteXPathHandle(tag)
    return locs
    
    